<?php
/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Plugins.php 3829 2005-11-01 12:41:09Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/Max.php';

/**
 * The place where to store plugins config and cache files
 */
define('MAX_PLUGINS_VAR', MAX_PATH . '/var/plugins');

/**
 * Write mode for folders inside MAX_PLUGINS_VAR
 */
define('MAX_PLUGINS_VAR_WRITE_MODE', 0755);

/**
 * File mask for plugins
 */
define('MAX_PLUGINS_FILE_MASK', '^.*/([a-zA-Z0-9\-_]*)/([a-zA-Z0-9\-_]*)\.plugin\.php$');

/**
 *
 * Max_Plugins is a class for factory the specific plugin from specific folder/package,
 * reading group of plugins from module/package, calling methods on plugins,
 * reading and writing plugin configuration files and plugin cache files and more.
 *
 * @package    MaxPlugins
 * @author     Radek Maciaszek <radek@m3.net>
 * @static
 */
class Max_Plugins
{
    var $module;
    var $package;
    var $name;
    
    /**
     * Factory plugin from specific module/package/plugin name
     *
     * @static 
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return object          Plugin object or false if any error occured
     *
     */
    function &factory($module, $package, $name = null)
    {
        if (!Max_Plugins::includePluginFile($module, $package, $name)) {
            return false;
        }
        if ($name === null) {
            $name = $package;
        }
        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        $obj = &new $className($module, $package, $name);
        $obj->module = $module;
        $obj->package = $package;
        $obj->name = $name;
        return $obj;
    }
    
    /**
     * Factory main module plugin. This method read the module configuration file
     * and (if module config exists and if config contain information about main plugin)
     * return main plugin for module (example: look at geotargeting)
     *
     * @static 
     * @param string $module     Module name (plugins/module folder name)
     * @param string $configKey  Name of configuration type which should
     *                           contain name of main plugin
     * @param string $omit       If configKey contain this variable it means
     *                           that main plugin is not defined
     *
     * @return object            Plugin object or false if any error occured
     *
     */
    function &factoryPluginByModuleConfig($module, $configKey = 'type', $omit = 'none')
    {
        // read module configuration file
        $conf = Max_Plugins::staticGetConfig($module);
        // get plugin type from configuration
        if (!isset($conf[$configKey])) {
            return false;
        } else {
            $packageName = explode(':', $conf[$configKey]);
            if (count($packageName) > 1) {
                $package = $packageName[0];
                $name = $packageName[1];
            } else {
                $package = $conf[$configKey];
                $name = null;
            }
        }
        if ($package == $omit) {
            // prevent from factory non existings class
            return null;
        }
        if (!empty($module) && !empty($package)) {
            return Max_Plugins::factory($module, $package, $name);
        } else {
            return false;
        }
    }
    
    /**
     * Include plugin file
     *
     * @static 
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return bool            True on success else false
     *
     */
    function includePluginFile($module, $package, $name = null)
    {
        if ($name === null) {
            $name = $package;
        }
        $fileName = MAX_PATH . "/plugins/{$module}/{$package}/{$name}.plugin.php";
        if (!file_exists($fileName)) {
            MAX::raiseError("Unable to include the {$fileName} file");
            return false;
        } else {
            include_once $fileName;
        }
        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        if (!class_exists($className)) {
            MAX::raiseError("Plugin file included but class '$className' doesn't exists");
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * This method include plugin file and call a static method on plugin class
     *
     * @static 
     * @param string $module        Module name (plugins/module folder name)
     * @param string $package       Package name (plugins/module/package folder namer)
     * @param string $name          Name of the php file which contains the plugin
     * @param string $staticMethod  Name of the php file which contains the plugin
     *
     * @return object          Plugin object or false if any error occured
     *
     */
    function &callStaticMethod($module, $package, $name = null, $staticMethod)
    {
        if ($name === null) {
            $name = $package;
        }
        if (!Max_Plugins::includePluginFile($module, $package, $name)) {
            return false;
        }
        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        $ret = call_user_func(array($className, $staticMethod));
        return $ret;
    }
    
    /**
     * Method read plugin config file
     *
     * @param string $module               Module name (plugins/module folder name)
     * @param string $package              Package name (plugins/module/package folder namer)
     * @param string $name                 Name of the php file which contains the plugin
     * @param bool   $processSections      Look at: http://uk.php.net/manual/en/function.parse-ini-file.php
     *                                     as one dimension array
     *
     * @return array                       Config array
     *
     */
    function staticGetConfig($module, $package = null, $name = null, $processSections = false, $copyDefaultIfNotExists = true)
    {
        $configFileName = Max_Plugins::getConfigFileName($module, $package, $name);
        $conf = Max_Plugins::getConfigByFileName($configFileName, $processSections, false);
        if ($conf !== false) {
            return $conf;
        } elseif ($copyDefaultIfNotExists) {
            Max_Plugins::copyDefaultConfig($module, $package, $name);
            $defaultConfigFileName = Max_Plugins::getConfigFileName($module, $package, $name, true);
            return Max_Plugins::getConfigByFileName($configFileName, $processSections, false);
        }
        MAX::raiseError("Config file '{$configFileName}' doesn't exists", MAX_ERROR_NOFILE);
        return false;
    }
    
    /**
     * Common plugin method for reading the specific plugin config file
     *
     * @static 
     * @param string $fileName         Module name (plugins/module folder name)
     * @param bool   $processSections  Look at: http://uk.php.net/manual/en/function.parse-ini-file.php
     * @param bool   $raiseErrors      If true raise pear error on failure
     *
     * @return array  Config array
     *
     */
    function getConfigByFileName($configFileName, $processSections = false, $raiseErrors = true)
    {
        if (file_exists($configFileName)) {
            $conf = parse_ini_file($configFileName, $processSections);
            if (isset($conf['realConfig'])) {
                if (ereg('.*\/(.*)\.plugin\.conf\.ini', $configFileName, $match = null)) {
                    $configFileName = str_replace($match[1], $conf['realConfig'], $configFileName);
                    return Max_Plugins::getConfigByFileName($configFileName, $processSections, $raiseErrors);
                } else {
                    return false;
                }
            }
            if (is_array($conf)) {
                return $conf;
            } else {
                return false;
            }
        } else {
            if ($raiseErrors) {
                MAX::raiseError("Config file '{$configFileName}' doesn't exists", MAX_ERROR_NOFILE);
            }
            return false;
        }
    }
    
    /**
     * Method build and return configuration file path for specific plugin/package/module
     *
     * @static 
     * @param string $module         Module name
     * @param string $package        Package name
     * @param string $name           Plugin name
     * @param bool   $defaultConfig  If true return path to default
     *                               config else to plugin config
     *
     * @return string    Path to the configuration file
     *
     */
    function getConfigFileName($module, $package = null, $name = null, $defaultConfig = false, $host = null)
    {
        if ($defaultConfig) {
            if(is_null($host)) {
                $host = 'default';
            }
            $startPath  = MAX_PATH . '/plugins/';
        } else {
            if(is_null($host)) {
                $host = Max_Plugins::getHostName();
            }
            $startPath  = MAX_PLUGINS_VAR . '/config/';
        }
        $configName = $host.'.plugin.conf.ini';
        if ($package === null) {
            $configPath = $module.'/';
        } elseif($name === null) {
            $configPath = $module.'/'.$package.'/';
        } else {
            $configPath = $module.'/'.$package.'/'.$name.'.';
        }
        
        return $startPath . $configPath . $configName;
    }
    
    /**
     * Copy default config file from plugins folder to var
     *
     * @param string $module               Module name (plugins/module folder name)
     * @param string $package              Package name (plugins/module/package folder namer)
     * @param string $name                 Name of the php file which contains the plugin
     *
     * @return bool  True on success
     *
     */
    function copyDefaultConfig($module, $package = null, $name = null)
    {
        $configFileName = Max_Plugins::getConfigFileName($module, $package, $name);
        $defaultConfigFileName = Max_Plugins::getConfigFileName($module, $package, $name, $default = true);
        if (file_exists($defaultConfigFileName)) {
            // create directory (if not exists)
            Max_Plugins::mkDirRecursive(dirname($configFileName), MAX_PLUGINS_VAR_WRITE_MODE);
            // copy file
            $ret = @copy($defaultConfigFileName, $configFileName);
            return $ret;
        }
        return false;
    }
    
    /**
     * Write config for plugin. Create also config "fake" files.
     *
     * @param array $aConfig   Array to save as a config
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return bool  True on success else false
     *
     */
    function writePluginConfig($aConfig, $module, $package = null, $name = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $configFileName = Max_Plugins::getConfigFileName($module, $package, $name);
        if (!file_exists($configFileName)) {
            Max_Plugins::copyDefaultConfig($module, $package, $name);
        }
        include_once 'Config.php';
        $c = new Config();
        $c->parseConfig($aConfig, 'phpArray');
        if (!$c->writeConfig($configFileName, 'inifile')) {
            return false;
        }
        // Create fake files
        $configHosts = array(
            $conf['webpath']['admin'],
            $conf['webpath']['delivery'],
            $conf['webpath']['deliverySSL']
        );
        $hostName = Max_Plugins::getHostName();
        foreach($configHosts as $host) {
            $url = @parse_url('http://' . $host);
            if (!isset($url['host'])) {
                return false;
            }
            $parsedHost = $url['host'];
            if ($parsedHost == $hostName) {
                continue;
            }
            // Create fake file for this host
            $configFileName = Max_Plugins::getConfigFileName($module, $package, $name, false, $parsedHost);
            $config = array('realConfig' => $hostName);
            $c = new Config();
            $c->parseConfig($config, 'phpArray');
            if (!$c->writeConfig($configFileName, 'inifile')) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Build and return class name for specific plugin
     *
     * @static 
     * @param string $module   Module name
     * @param string $package  Package name
     * @param string $name     Plugin name
     *
     * @return string          Plugin class name
     *
     */
    function getPluginClassName($module, $package, $name = null)
    {
        if ($name === null) {
            $name = $package;
        }
        $className = "Plugins_" . ucfirst($module) . '_' . ucfirst($package) . '_' . ucfirst($name);
        return $className;
    }
    
    /**
     * Return the list of files from selected folder
     *
     * @static 
     * @param string $folder                   Folder path
     * @param string $onlyPluginNameAsIndex    If true the key for plugins is "pluginName" else
     *                                         key format is "packageName:pluginName"
     * @param int|bool $recursive              How deep should we look for plugins,
     *                                         (default is 1). If true read recursively all the files also from subdirectories
     *
     * @return array                           List of files
     */
    function &getPlugins($module, $package = null, $onlyPluginNameAsIndex = true, $recursive = 1)
    {
        $plugins = array();
        $pluginFiles = Max_Plugins::getPluginsFiles($module, $package, $recursive);
        foreach ($pluginFiles as $key => $pluginFile) {
            $pluginInfo = explode(':', $key);
            if (count($pluginInfo) > 1) {
                if ($onlyPluginNameAsIndex) {
                    $plugins[$pluginInfo[1]] = Max_Plugins::factory($module, $pluginInfo[0], $pluginInfo[1]);
                } else {
                    $plugins[$key] = Max_Plugins::factory($module, $pluginInfo[0], $pluginInfo[1]);
                }
            }
        }
        return $plugins;
    }
    
    /**
     * Build a plugin folder path and return array with all the plugin files from this folder
     *
     * @param string $module      Name of the module
     * @param string $package     Name of the package
     * @param bool   $recursive  Search plugins in subdirectories
     *
     * @return array              List of files
     */
    function getPluginsFiles($module, $package = null, $recursive = 1)
    {
        $pluginsDir = MAX_PATH.'/plugins';
        if (!empty($package)) {
            $dir = $pluginsDir.'/'.$module.'/'.$package;
        } else {
            $dir = $pluginsDir.'/'.$module;
        }
        return Max_Plugins::getPluginsFilesFromFolder($dir, $recursive);
    }
    
    /**
     * Return the list of files from selected folder which match the specified criteria (file mask)
     *
     * @static 
     * @param string $folder              Folder path
     * @param integer|bool $recursive  If true read all the files also from subdirectory
     *                                    If integer read as many subdirectories
     *
     * @return array                      List of files
     */
    function getPluginsFilesFromFolder($folder, $recursive = 1)
    {
        include_once MAX_PATH . '/lib/max/FileScanner.php';
        $fs = new Max_FileScanner();
        $fs->addFileTypes(array('php','inc'));
        $fs->setFileMask(MAX_PLUGINS_FILE_MASK);
        $fs->addDir($folder, $recursive);
        return $fs->getAllFiles();
    }

    /**
     * Common method to run one method on group of plugins.
     * 
     * TODO: add a possibility to pass additional parameters to $callbackMethod
     *
     * @static 
     * @param array $plugins          Array of plugins
     * @param string $callbackMethod  This method will be executed on every plugin
     *
     * @return array                  Array of results after calling $callbackMethod
     *                                on every plugin
     */
    function &callOnPlugins(&$plugins, $callbackMethod)
    {
        if(!is_array($plugins)) {
            MAX::raiseError('Bad argument', MAX_ERROR_INVALIDARGS);
        }
        $ret = array();
        foreach ($plugins as $key => $plugin) {
            if (is_callable(array($plugin, $callbackMethod))) {
                $ret[$key] = call_user_func(array($plugins[$key], $callbackMethod));
            }
        }
        return $ret;
    }
    
    /**
     * Test if a cache is available and (if yes) return it
     *
     * @static 
     * @param string  $id       Cache id
     * @param string  $module   Module name (plugins/module folder name)
     * @param string  $package  Package name (plugins/module/package folder namer)
     * @param string  $name     Name of the php file which contains the plugin
     * @param bool    $doNotTestCacheValidity   If set to true, the cache validity won't be tested
     * @param array   $options  Options - see Cache_Lite()
     *
     * @return mixed           Data of the cache (or false if no cache available)
     *
     */
    function getCacheForPluginById($id, $module, $package, $name = null, $doNotTestCacheValidity = true, $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = Max_Plugins::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        $data = $cache->get($id, $name, $doNotTestCacheValidity);
        return $data;
    }

    /**
     * Save data in plugin cache file
     *
     * @static 
     * @param string $data     Data to put in cache (can be another type than strings if automaticSerialization is on)
     * @param string $id       Cache id
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     * @param array  $options  Options - see Cache_Lite()
     *
     * @return bool            True if no problem
     *
     */
    function saveCacheForPlugin($data, $id, $module, $package, $name = null, $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = Max_Plugins::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        return $cache->save($data, $id, $name);
    }

    /**
     * Delete plugin cache
     *
     * @static 
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     * @param string $mode     Flush cache mode : 'old', 'ingroup', 'notingroup'
     * @param array  $options  Options - see Cache_Lite()
     *
     * @return bool            True if no problem
     *
     */
    function cleanPluginCache($module, $package, $name = null, $mode = 'ingroup', $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = Max_Plugins::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        return $cache->clean($name, $mode);
    }

    /**
     * Build PEAR Cache_Lite options for plugin from specified module/package.
     * If cacheDir doesn't exists this method also create it.
     *
     * @param string $module    Module name
     * @param string $package   Package name
     *
     * @see getCacheForPluginById()
     * @see saveCacheForPlugin()
     *
     * @return array    Array with cache options for PEAR Cache_Lite class
     */
    function prepareCacheOptions($module, $package, $cacheDir = null, $cacheExpire = null)
    {
        $options = array(
            'cacheDir' => (is_null($cacheDir)) ? MAX_PLUGINS_CACHE . '/' . $module . '/' . $package . '/cache/' : $cacheDir,
            'lifeTime' => (is_null($cacheExpire)) ? -1 : $cacheExpire,
            'automaticSerialization' => true,
        );
        if (!file_exists($options['cacheDir'])) {
            if (is_writable(MAX_PLUGINS_CACHE)) {
                Max_Plugins::mkDirRecursive($options['cacheDir'], MAX_PLUGINS_VAR_WRITE_MODE);
            } else {
                Max::raiseError('Folder: "' . MAX_PLUGINS_CACHE . '" is not writeable.');
            }
        }
        return $options;
    }
    
    /**
     * Util method - for creating recursively $path folder. Helpfull for
     * creating folders for cache and for config files
     *
     * @param string $path    Folder path to create
     * @param int    $mode    Mode (access permissions)
     */
    function mkDirRecursive($path, $mode = null)
    {
        if (is_dir($path)) {
            return true;
        } else {
            $ret1 = Max_Plugins::mkDirRecursive(dirname($path));
            $ret2 = @mkdir($path, $mode);
            if ($ret2) {
                @chmod($path, MAX_PLUGINS_VAR_WRITE_MODE);
            }
            return $ret1 && $ret2;
        }
    }
    
    /**
     * Return host name
     *
     * @return string Host name
     * @access private
     */
    function getHostName()
    {
        if (is_null($_SERVER['HTTP_HOST'])) {
            if (is_null($GLOBALS['argv'][1])) {
                MAX::raiseError(MAX_PRODUCT_NAME .
                    " was called via the command line, but had no host as a parameter.\n", PEAR_ERROR_DIE);
                return false;
            }
            $host = $GLOBALS['argv'][1];
        } else {
            if (!empty($_SERVER['HTTP_HOST'])) {
                $host = $_SERVER['HTTP_HOST'];
            } else {
                $host = $_SERVER['SERVER_NAME'];
            }
        }
        return $host;
    }
}

?>
